﻿using Models.Events;
using Models.Peristance;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Runtime.CompilerServices;
using System.Runtime.Serialization;
using System.Text;
using System.Threading.Tasks;


namespace Models.Jeux.Plateau
{
	[DataContract]
	public class Partie:INotifyPropertyChanged
	{

		/// <summary>
		/// Evénement affichant le gagnant de la partie
		/// </summary>
		public event EventHandler<ShowingWinnerEventArgs> ShowingWinner;

		public event PropertyChangedEventHandler? PropertyChanged;

		void OnPropertyChanged([CallerMemberName] string propertyName = null)
			=> PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(propertyName));

		protected virtual void OnShowingWinner(Joueur j)
		{
			ShowingWinner?.Invoke(this, new ShowingWinnerEventArgs(j));
		}

		/// <summary>
		/// Constructeur de la classe partie,crée un nouveau plateau et assigne les couleurs au joueurs
		/// </summary>
		/// <param name="J1">Objet J1 de type Joueur correspondant au joueur numéro 1 de la partie</param>
		/// <param name="J2">Objet J2 de type Joueur correspondant au joueur numéro 2 de la partie</param>
		public Partie(Joueur J1, Joueur J2)
		{
			Plateau = new Plateau();
			
			Joueur1 = J1;
			Joueur2 = J2;
			JoueurCourrant = J1;
			JoueurNonCourant = J2;
			Joueur1.CouleurDePion = Couleur.BLANC;
			Joueur2.CouleurDePion = Couleur.NOIR;
		}

		public Partie()
		{
			Plateau = new Plateau();
			Joueur1 = new Joueur("GEORGE",Couleur.BLANC);
			Joueur2 = new Joueur("TITI",Couleur.NOIR);
			JoueurCourrant = Joueur1;
			JoueurCourrant=Joueur2;
			
		}




		/// <summary>
		/// Crée le getter et le setter de l'attribut plateau de partie
		/// </summary>
		[DataMember]
		public Plateau Plateau
		{
			get
			{
				return plateau;
			}
			set
			{
				plateau = value;
				OnPropertyChanged();

			}
		}

		private Plateau plateau;

		/// <summary>
		/// Crée le getter et le setter de l'attribut Joueur1 de partie
		/// </summary>
		[DataMember]
		public Joueur Joueur1
		{
			get
			{
				return joueur1;
			}
			set
			{ 
				joueur1 = value; 
				OnPropertyChanged();
			}
		}

		private Joueur joueur1;

		/// <summary>
		/// Crée le getter et le setter de l'attribut Joueur2 de partie
		/// </summary>
		[DataMember]
		public Joueur Joueur2
		{
			get
			{
				return joueur2;
			}
			set
			{
				joueur2 = value;
				OnPropertyChanged();
			}
		}

		private Joueur joueur2;


		/// <summary>
		/// Crée le getter et le setter de l'attribut JoueurCourrant de partie
		/// </summary>
		[DataMember]
		public Joueur JoueurCourrant
		{
			get
			{
				return joueurCourrant;
			}
			set
			{
				joueurCourrant = value;
				OnPropertyChanged();
			}
		}

		private Joueur joueurCourrant;

		[DataMember]
		public Case? CaseCibleCourant
		{
			get
			{ 
				return caseCibleCourant;
			}
			set
			{
				caseCibleCourant = value;

			}
		}

		private Case? caseCibleCourant=null;

		[DataMember]
		public Pion? PionCourant
		{
			get
			{
				return pionCourant;
			}
			set
			{
				pionCourant = value;
			}
		}


		private Pion? pionCourant=null;

		[DataMember]
		public List<Case> ListeManger
		{
			get
			{ 
				return listeManger;
			}
			set 
			{ 
				listeManger = value;
				OnPropertyChanged();
			}
	
		}
		private List<Case> listeManger = new List<Case>();


		public Joueur JoueurNonCourant
		{
			get
			{
				return joueurNonCourant;
			}
			set
			{
				joueurNonCourant = value; 
				OnPropertyChanged();

			}
		}
		private Joueur joueurNonCourant;

		/// <summary>
		/// Permet de savoir si quelqu'un a gagné la partie et dis qui est le vainqueur , chaque fois que l'on prend un 
		/// pion adverse, on gagne un point et celui a 20 gagne
		/// </summary>
		/// <returns>Renvoie un int correspondant au numéro du joueur et 0 si personne n'a encore gagné</returns>
		public int Agagner()
		{
			if (Joueur1.NbPions == 20)
			{
				return 1;
			}
			else
			{
				if (Joueur2.NbPions == 20)
				{
					return 2;
				}
			}
			return 0;

		}

		public void JouerTour(Pion p, Case c)
		{
			
			bool verif = false;
			bool verifPion = false, verifCible = false;
			List <Case> verifAffiler =new List<Case>();
			int nbPions = Plateau.ListePions.Count;
			
			if (p != null && c != null && Agagner() == 0)
			{

			   if(ListeManger.Count > 0)
				{
					foreach (Case c2 in  ListeManger)
					{
						if(c2==c)
						{
							verifCible = true;
						}
						if(p.Pos==c2)
						{
							verifPion = true;
						}

					}
				}
			   if(ListeManger.Count == 0 || (verifPion==true && verifCible==true))
				{
                    verif = Plateau.deplacer(p, c, JoueurCourrant);
                    PionCourant = null;
                    CaseCibleCourant = null;
					verifPion=false;
					verifCible=false;
                    ListeManger = Plateau.PossibiliteManger(JoueurNonCourant);

                    if (nbPions != Plateau.ListePions.Count)
                    {
                        nbPions = Plateau.ListePions.Count;
                        verifAffiler = Plateau.MangerAffiler(JoueurCourrant, c);
                        if (verifAffiler.Count != 0)
                        {
                            Pion p2 = Plateau.recherchePion(c);
                            if (p2 != null)
                            {
                                JouerTour(p2, verifAffiler[0]);
                            }

                        }
                    }

                    if (verif == true && verifAffiler.Count == 0)
                    {
                        if (JoueurCourrant == Joueur1)
                        {
                            JoueurCourrant = Joueur2;
                            JoueurNonCourant = Joueur1;

                        }

                        else
                        {
                            JoueurCourrant = Joueur1;
                            JoueurNonCourant = Joueur2;
                        }

                    }
                }
                PionCourant = null;
                CaseCibleCourant = null;

            }
		}





		/// <summary>
		/// Appelle l'événement affichant le gagnant de la partie
		/// </summary>
		public void AfficherGagnant()
		{
			Joueur j = new Joueur();
			OnShowingWinner(j);
		}
	}

}
