﻿using System.ComponentModel;
using System.Runtime.CompilerServices;
using System.Runtime.Serialization;

namespace Models.Jeux.Plateau

{
    [DataContract]
    public class Case:INotifyPropertyChanged
    {

        public event PropertyChangedEventHandler? PropertyChanged;

        void OnPropertyChanged([CallerMemberName] string propertyName = null)
            => PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(propertyName));

        /// <summary>
        /// Constructeur de la classe Case
        /// </summary>
        /// <param name="couleur">Objet couleur de type Couleur correspondant à la couleur de la case</param>
        /// <param name="X">Objet X de type int correspondant à la colone de la case</param>
        /// <param name="Y">Objet Y de type int correspondant à la ligne de la case</param>
        public Case(Couleur couleur, int X, int Y)
        {
            CouleurCase = couleur;
            PosX = X;
            PosY = Y;
        }



        /// <summary>
        /// Créé le getter et le setter de l'attribut Couleur Case
        /// </summary>
        [DataMember]
        public Couleur CouleurCase
        {
            get
            {
                return couleurCase;
            }

            set
            {
                couleurCase = value;
                OnPropertyChanged();
            }
        }
        private Couleur couleurCase;

        /// <summary>
        /// Créé le getter et le setter de l'attribut PosX et vérifie que la valeur de celui-ci est
        /// comprise entre 0 et 9
        /// </summary>
        [DataMember]
        public int PosX
        {
            get
            {
                return posX;
            }
            set
            {
                if (value < 0)
                {
                    posX = 0;
                }
                else
                {
                    if (value > 9)
                    {
                        posX = 9;
                    }
                    else
                        posX = value;
                }


            }
        }
        private int posX;

        /// <summary>
        /// Créé le getter et le setter de l'attribut PosY et vérifie que la valeur de celui-ci est
        /// comprise entre 0 et 9
        /// </summary>
        [DataMember]
        public int PosY
        {
            get
            {
                return posY;
            }
            set
            {
                if (value < 0)
                {
                    posY = 0;
                }
                else
                {
                    if (value > 9)
                    {
                        posY = 9;
                    }
                    else
                        posY = value;
                }
            }
        }
        private int posY;

        /// <summary>
        /// Créé le getter et le setter de l'attribut Pion
        /// </summary>
        [DataMember]
        public bool Pion
        {
            get
            {
                return pion;
            }
            set
            {
                pion = value;
                OnPropertyChanged();
            }
        }
        private bool pion = false;
        
        public Pion? Pion2
        {
            get
            {
                return pion2;
            }
            set
            {
                pion2 = value;
                OnPropertyChanged();
            }
        }

        private Pion? pion2 = null;


        public override bool Equals(object? obj)
        {
            if (obj == null || !(obj is Case))
                return false;
            else
            {
                Case c = (Case)obj;
                return (c.posX == this.posX) && (c.posY == this.posY) && (c.couleurCase == this.couleurCase);
            }
                
        }

        /// <summary>
        /// Affiche une case
        /// </summary>
        /// <returns>Une phrase quidit la couleur de la case et ses coordonnées</returns>
        public override string ToString()
        {
            return $"Cette case est {CouleurCase} et ses coordonnées sont X={PosX} Y={PosY}";
        }

    }
}
